#include "LoRaLib.h"

LoRa::LoRa(Chip ch, int nss, float freq, uint32_t bw, uint8_t sf, uint8_t cr, int dio0, int dio1, uint8_t syncWord) {
  dataRate = 0;
  lastPacketRSSI = 0;
  lastPacketSNR = 0;
  
  switch(ch) {
    case CH_SX1272:
      _mod = new SX1272(nss, freq, bw, sf, cr, dio0, dio1, syncWord);
      break;
    case CH_SX1273:
      _mod = new SX1273(nss, freq, bw, sf, cr, dio0, dio1, syncWord);
      break;
    case CH_SX1276:
      _mod = new SX1276(nss, freq, bw, sf, cr, dio0, dio1, syncWord);
      break;
    case CH_SX1277:
      _mod = new SX1277(nss, freq, bw, sf, cr, dio0, dio1, syncWord);
      break;
    case CH_SX1278:
      _mod = new SX1278(nss, freq, bw, sf, cr, dio0, dio1, syncWord);
      break;
    case CH_SX1279:
      _mod = new SX1279(nss, freq, bw, sf, cr, dio0, dio1, syncWord);
      break;
  }
}

uint8_t LoRa::begin(uint16_t addrEeprom) {
  DEBUG_BEGIN(9600);
  DEBUG_PRINTLN();
  
  #ifdef ESP32
    if(!EEPROM.begin(9)) {
      DEBUG_PRINTLN("Unable to initialize EEPROM");
      return(ERR_EEPROM_NOT_INITIALIZED);
    }
  #endif
  
  _addrEeprom = addrEeprom;
  
  bool hasAddress = false;
  for(uint16_t i = 0; i < 8; i++) {
    if(EEPROM.read(_addrEeprom + i) != 255) {
      hasAddress = true;
      break;
    }
  }
  
  if(!hasAddress) {
    randomSeed(analogRead(5));
    generateLoRaAdress();
  }
  
  DEBUG_PRINTLN("LoRa node address string: ");
  for(uint8_t i = 0; i < 8; i++) {
    _address[i] = EEPROM.read(i);
    #ifdef LORALIB_DEBUG
      Serial.print(_address[i], HEX);
      if(i < 7) {
        Serial.print(":");
      } else {
        Serial.println();
      }
    #endif
  }
  
  return(_mod->begin());
}

uint8_t LoRa::transmit(Packet& pack) {
  char buffer[256];
  
  memcpy(buffer, pack.source, 8);
  memcpy(buffer + 8, pack.destination, 8);
  memcpy(buffer + 16, pack.data, pack.length - 16);
  
  return(_mod->tx(buffer, pack.length));
}

uint8_t LoRa::receive(Packet& pack) {
  char buffer[256];
  uint32_t startTime = millis();
  
  uint8_t status = _mod->rxSingle(buffer, &pack.length);
  if(status != ERR_NONE) {
    return(status);
  }
  
  memcpy(pack.source, buffer, 8);
  memcpy(pack.destination, buffer + 8, 8);
  
  delete[] pack.data;
  pack.data = new char[pack.length - 15];
  memcpy(pack.data, buffer + 16, pack.length - 16);
  pack.data[pack.length-16] = 0;
  
  uint32_t elapsedTime = millis() - startTime;
  dataRate = (pack.length*8.0)/((float)elapsedTime/1000.0);
  lastPacketRSSI = _mod->getLastPacketRSSI();
  lastPacketSNR = _mod->getLastPacketSNR();
  
  return(status);
}

uint8_t LoRa::scanChannel() {
  return(_mod->runCAD());
}

uint8_t LoRa::sleep() {
  return(_mod->setMode(0b00000000));
}

uint8_t LoRa::standby() {
  return(_mod->setMode(0b00000001));
}

uint8_t LoRa::setBandwidth(uint32_t bw) {
  return(_mod->setBandwidth(bw));
}

uint8_t LoRa::setSpreadingFactor(uint8_t sf) {
  return(_mod->setSpreadingFactor(sf));
}

uint8_t LoRa::setCodingRate(uint8_t cr) {
  return(_mod->setCodingRate(cr));
}

uint8_t LoRa::setFrequency(float freq) {
  return(_mod->setFrequency(freq));
}

uint8_t LoRa::setSyncWord(uint8_t syncword) {
  return(_mod->setSyncWord(syncword));
}

uint8_t LoRa::setOutputPower(int8_t power) {
  return(_mod->setOutputPower(power));
}

void LoRa::generateLoRaAdress() {
  for(uint8_t i = _addrEeprom; i < (_addrEeprom + 8); i++) {
    EEPROM.write(i, (uint8_t)random(0, 256));
  }
}
